package Demo;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.lang.Thread.UncaughtExceptionHandler;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Date;

import org.python.core.PyCode;
import org.python.core.PyException;
import org.python.core.PyList;
import org.python.core.PyObject;
import org.python.core.PySystemState;
import org.python.modules._py_compile;
import org.python.util.PythonInterpreter;



public class jython_test {

	static ThreadSafePrintStream outStream = new ThreadSafePrintStream(
			System.out);

	public static void main(String[] args) throws IOException,
			InterruptedException {
		final int NUM_THREADS = 1;

		PrintStream orig = System.out;

		PrintStream[] usersSysOut = new PrintStream[NUM_THREADS];
		Thread[] threads = new Thread[NUM_THREADS];
		ByteArrayOutputStream [] userSysOut = new ByteArrayOutputStream[NUM_THREADS];

		Date d1 = new Date();
		
		for (int i = 0; i < NUM_THREADS; i++) {
			userSysOut[i] = new ByteArrayOutputStream();
			usersSysOut[i] = new PrintStream(userSysOut[i]);
		}

		for (int i = 0; i < (double) NUM_THREADS / 2; i++) {
			JythonThread1 j1 =  new JythonThread1("JythonThread[" + i + "]", outStream);
			threads[i] = new Thread(null, j1, "JythonThread1-"+i, 1024*1024);
			outStream.addStream(usersSysOut[i], threads[i]);
		}

		for (int i = (int) Math.round((double) NUM_THREADS / 2); i < NUM_THREADS; i++) {
			JythonThread2 j2 =  new JythonThread2("JythonThread[" + i + "]", outStream);
			threads[i] = new Thread(null, j2, "JythonThread2-"+i, 1024*1024);
			outStream.addStream(usersSysOut[i], threads[i]);			
		}
		
		System.setOut(outStream);

		for (int i = 0; i < NUM_THREADS; i++) {
			threads[i].start();
		}


		for (int i = 0; i < NUM_THREADS; i++) {
			threads[i].join();
		}
		
		Date d2 = new Date();
		
		System.setOut(orig);		
	
		for (int i = 0; i < NUM_THREADS; i++) {
			System.out.println("JythonThread[" + i + "]:\n"
					+ userSysOut[i].toString());
			outStream.removeStream(threads[i]);
		}
		
		System.out.println("Time to run for " + NUM_THREADS + " threads: " + (d2.getTime() - d1.getTime()) + " ms");

	}
}

class JythonThread1 implements Runnable {
	private String name;
	private PrintStream out;
	private PythonInterpreter interp;

	public JythonThread1(String name, PrintStream out) {
		// Pass name to Thread layer.
		this.out = out;
		PySystemState p = new PySystemState(); 
		p.setrecursionlimit(10);
		interp = new PythonInterpreter(null, p);
		interp.setOut(out);
	}

	void runScript(String name) throws IOException {
		try {			
			File f = new File(name);
			FileInputStream fis = new FileInputStream(f);
			InputStreamReader s = new InputStreamReader(fis);
			PyCode code = interp.compile(s);
			interp.exec(code);
		} catch (PyException e) {					
	    	e.printStackTrace(out);
		} catch (Exception e) {
			e.printStackTrace(out);
		}
	}

	public void run() {
		try {
			runScript("pfile25.py");
		} catch (IOException e) {
		//	System.setOut(System.out);
			e.printStackTrace();
		}
	}
}

class JythonThread2 implements Runnable {

	PrintStream out;
	PythonInterpreter interp;

	public JythonThread2(String name, PrintStream out) {
		// Pass name to Thread layer.
		this.out = out;
		PySystemState p = new PySystemState();
		p.setrecursionlimit(10);		
		interp = new PythonInterpreter(null, p);
		interp.setOut(out);
	}

	void runScript(String name) throws IOException {
		try {

			File f = new File(name);
			FileInputStream fis = new FileInputStream(f);
			InputStreamReader s = new InputStreamReader(fis);
			PyCode code = interp.compile(s);
			interp.exec(code);
			PyObject r1 = interp.eval("increment(10, 0)");
			PyObject r2 = interp.eval("permutation([1,2,3])");			
			System.out.println(r1.toString());
			System.out.println(r2.toString());						
		} catch (Exception e) {
			e.printStackTrace(out);
		}
	}

	public void run() {
		try {
			runScript("pfile4.py");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace(out);
		}
	}

}
